package db

import (
	"applet/app/db/gim/model"
	"applet/app/utils"
	"applet/app/utils/logx"
	"errors"
	"fmt"
	"reflect"
	"xorm.io/xorm"
)

// BatchSelectUsers 批量查询数据 TODO::和下面的方法重复了，建议采用下面的 `UserFindByParams` 方法
func BatchSelectUsers(Db *xorm.Engine, params map[string]interface{}) (*[]model.User, error) {
	var UserData []model.User
	if err := Db.In(utils.AnyToString(params["key"]), params["value"]).
		Find(&UserData); err != nil {
		return nil, logx.Warn(err)
	}
	return &UserData, nil
}

// UserInsert 插入单条数据
func UserInsert(Db *xorm.Engine, User *model.User) (int64, error) {
	_, err := Db.InsertOne(User)
	if err != nil {
		return 0, err
	}
	return User.Id, nil
}

// BatchAddUsers 批量新增数据
func BatchAddUsers(Db *xorm.Engine, UserData []*model.User) (int64, error) {
	affected, err := Db.Insert(UserData)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func GetUserCount(Db *xorm.Engine) int {
	var User model.User
	session := Db.Where("")
	count, err := session.Count(&User)
	if err != nil {
		return 0
	}
	return int(count)
}

// UserDelete 删除记录
func UserDelete(Db *xorm.Engine, id interface{}) (int64, error) {
	if reflect.TypeOf(id).Kind() == reflect.Slice {
		return Db.In("id", id).Delete(model.User{})
	} else {
		return Db.Where("id = ?", id).Delete(model.User{})
	}
}

// UserUpdate 更新记录
func UserUpdate(Db *xorm.Engine, id interface{}, User *model.User, forceColums ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColums != nil {
		affected, err = Db.Where("id=?", id).Cols(forceColums...).Update(User)
	} else {
		affected, err = Db.Where("id=?", id).Update(User)
	}
	if err != nil {
		return 0, err
	}
	return affected, nil
}

// UserGetOneByParams 通过传入的参数查询数据(单条)
func UserGetOneByParams(Db *xorm.Engine, masterId string, params map[string]interface{}) (*model.User, error) {
	var m model.User
	var query = fmt.Sprintf("%s =?", params["key"])
	if has, err := Db.Where(query, params["value"]).And("master_id = ?", masterId).Get(&m); err != nil || has == false {
		return nil, logx.Error(err)
	}
	return &m, nil
}

// UserFindByParams 通过传入的参数查询数据(多条)
func UserFindByParams(Db *xorm.Engine, params map[string]interface{}) (*[]model.User, error) {
	var m []model.User
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if params["key"] == nil {
		//查询全部数据
		err := Db.Find(&m)
		if err != nil {
			return nil, logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Find(&m)
			if err != nil {
				return nil, logx.Error(err)
			}
			return &m, nil
		}

	}
}

// UserFindByLevel 查询等级数据(多条)
//func UserFindByLevel(level int) (*[]model.User, error)  {
//	var m []model.User
//
//}

// 根据经纬度获取网点
//  lng float64 经度
//  lat float64 维度
func GetSelectedSiteWithLatLng(engine *xorm.Engine, lng, lat float64, page, pageSize string, scopeOfBranches int) (*[]model.User, error) {
	var regions []model.User
	where := `sqrt( ( (( %f - longitude)*PI()*12656*cos((( %f +latitude)/2)*PI()/180)/180) * (( %f - longitude)*PI()*12656*cos (((%f+latitude)/2)*PI()/180)/180) ) + ( ((%f-latitude)*PI()*12656/180) * ((%f-latitude)*PI()*12656/180) ) ) < ` + utils.IntToStr(scopeOfBranches)
	where = fmt.Sprintf(where, lng, lat, lng, lat, lat, lat)
	if page == "" && pageSize == "" {
		err := engine.Where(where).Find(&regions)
		if err != nil {
			return nil, err
		}
		return &regions, nil
	}
	p := utils.StrToInt(page)      // 页码
	ps := utils.StrToInt(pageSize) // 每页记录数
	err := engine.Where(where).Limit(ps, (p-1)*ps).Find(&regions)
	if err != nil {
		return nil, err
	}
	return &regions, nil
}

// 根据经纬度获取最近的网点
//  lng float64 经度
//  lat float64 维度
func GetSelectedSiteWithLatLngNearClose(engine *xorm.Engine, lng, lat float64, scopeOfBranches int) (*model.User, error) {
	var region model.User
	where := `sqrt( ( (( %f - longitude)*PI()*12656*cos((( %f +latitude)/2)*PI()/180)/180) * (( %f - longitude)*PI()*12656*cos (((%f+latitude)/2)*PI()/180)/180) ) + ( ((%f-latitude)*PI()*12656/180) * ((%f-latitude)*PI()*12656/180) ) ) < ` + utils.IntToStr(scopeOfBranches)
	where = fmt.Sprintf(where, lng, lat, lng, lat, lat, lat)
	sel := `regional_agent_region.*, sqrt( ( (( %f - longitude)*PI()*12656*cos((( %f +latitude)/2)*PI()/180)/180) * (( %f - longitude)*PI()*12656*cos (((%f+latitude)/2)*PI()/180)/180) ) + ( ((%f-latitude)*PI()*12656/180) * ((%f-latitude)*PI()*12656/180) ) ) AS km`
	sel = fmt.Sprintf(sel, lng, lat, lng, lat, lat, lat)
	isHas, err := engine.Select(sel).
		Table("regional_agent_region").Where(where).Get(&region)
	if err != nil {
		return nil, err
	}
	if !isHas {
		return nil, nil
	}

	return &region, nil
}

// 根据总后台的区id获取代理网点
func GetSiteListWithWebSiteRegionId(session *xorm.Session, websiteRegionId string) (*model.User, error) {
	var regionD *model.User
	isExist, err := session.Table("regional_agent_region").
		Where("website_region_id = ?", websiteRegionId).
		Get(&regionD)
	if err != nil {
		return nil, err
	}
	if !isExist {
		return nil, nil
	}

	var regionSite *model.User
	isExist, err = session.Table("regional_agent_region").
		Where("district_id = ?", regionD.Id).Get(&regionSite)
	if err != nil {
		return nil, err
	}
	if !isExist {
		return regionD, nil
	}
	return regionSite, nil
}
